/* 555-a-like
   Adjustable Squarewave Generator
   Slider2732 December 2016
   Alternative Adafruit library for OLED screen
   
   Features:
   1- near 100% Duty Cycle
   1-14Hz
   Practically infinite adjustment of frequency & duty cycle
   Pin 13 as digital High/Low output - most Arduino's have an LED there
   Screen output, if required.

   This version - Adafruit library
   Add Adafruit SSD1306, Adafruit GFX and elapsedMillis
   from within IDE - Sketch/Include library/Manage libraries
   
   December 27th  10:35am
*/


// Variables and OLED 128x64 I2C library

#include <SPI.h>
#include <Wire.h>
#include <elapsedMillis.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>
#include <Fonts/FreeSerif9pt7b.h>    
#define Output_Pin 13  
#define OLED_RESET 4
Adafruit_SSD1306 display(OLED_RESET);

int Freq_Pot = A0;
int Frequency = 0;
int Duty_Pot = A1;
int Duty_Cycle = 0;
int millis_old = 0;
int Freq = 0;
int count = 0;
int On = 0;


elapsedMillis timeElapsed; 
unsigned int interval = 1000;

void setup()
{
  pinMode(Output_Pin, OUTPUT);   // set the output pin as output
  display.begin(SSD1306_SWITCHCAPVCC, 0x3C);
 
  display.invertDisplay(false);  // invert the display with 'true'
  display.setTextSize(0);
  display.setTextColor(WHITE);
  display.clearDisplay();
}


void loop()
{
  Freq = analogRead(Freq_Pot);
  int Freq_Read = (Freq, 0, 1023, 0, 100);  // convert to percentage
  On = analogRead(Duty_Pot);
  int Duty_Read = map(On, 0, 1023, 0, 100); // convert to percentage
  Duty_Cycle = Duty_Read;

  if (Duty_Cycle >= 95)
  { Duty_Cycle = 95; }  // slight delay due to running code !
  
  interval = Freq / 10;
  Freq_Read = interval;
  interval = interval *20; 
  interval = interval *5.12;      // full cycle

  draw();

  digitalWrite(Output_Pin, HIGH);           // turn the LED on (this is the MOSFET driver output HIGH)
  delay((Duty_Cycle * Freq_Read) ); 
  digitalWrite(Output_Pin, LOW);            // turn the LED off
  delay(((100 - Duty_Cycle) * Freq_Read) ); // the remainder
}


void draw()
{
  display.setFont(&FreeSerif9pt7b);
  display.clearDisplay();
  display.setCursor(0,14);
  display.print("555-A-Like");
  display.drawLine(0,18, 84,18, WHITE); 
  display.setCursor(0,40);
  display.print("Cycle: ");  
  display.setCursor(56,40); 
  display.print (interval);
  display.setCursor(106,40);
  display.print ("ms");
  display.setCursor(0,60);
  display.print("Duty: ");
  display.setCursor(56,60);
  display.print (Duty_Cycle);
  display.setCursor(104,60);
  display.print ("%");
  display.display();
}

